# -*- coding: utf-8 -*-
"""
Created on 2021-03-10 17:28:36
---------
@summary:
---------
@author: Boris
"""

from feapder import Item


class SpiderDataItem(Item):
    """
    This class was generated by feapder.
    command: feapder create -i spider_data.
    """

    def __init__(self, *args, **kwargs):
        # self.id = None
        self.title = None
